@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2006-2007 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -23,6 +23,7 @@
 #define __GPM_GRAPH_WIDGET_H__
 
 #include <gtk/gtk.h>
+#include "gpm-array.h"
 
 G_BEGIN_DECLS
 
@@ -33,36 +34,15 @@ G_BEGIN_DECLS
 #define GPM_IS_GRAPH_WIDGET_CLASS(obj)	(G_TYPE_CHECK_CLASS_TYPE ((obj), EFF_TYPE_GRAPH_WIDGET))
 #define GPM_GRAPH_WIDGET_GET_CLASS	(G_TYPE_INSTANCE_GET_CLASS ((obj), GPM_TYPE_GRAPH_WIDGET, GpmGraphWidgetClass))
 
-#define GPM_GRAPH_WIDGET_COLOUR_CHARGING	GPM_GRAPH_WIDGET_COLOUR_BLUE
-#define GPM_GRAPH_WIDGET_COLOUR_DISCHARGING	GPM_GRAPH_WIDGET_COLOUR_DARK_RED
-#define GPM_GRAPH_WIDGET_COLOUR_CHARGED		GPM_GRAPH_WIDGET_COLOUR_GREEN
-#define GPM_GRAPH_WIDGET_LEGEND_SPACING		19
+#define GPM_COLOUR_CHARGING			GPM_COLOUR_BLUE
+#define GPM_COLOUR_DISCHARGING			GPM_COLOUR_DARK_RED
+#define GPM_COLOUR_CHARGED			GPM_COLOUR_GREEN
+#define GPM_GRAPH_WIDGET_LEGEND_SPACING		17
 
 typedef struct GpmGraphWidget		GpmGraphWidget;
 typedef struct GpmGraphWidgetClass	GpmGraphWidgetClass;
 typedef struct GpmGraphWidgetPrivate	GpmGraphWidgetPrivate;
 
-typedef enum {
-	GPM_GRAPH_WIDGET_COLOUR_DEFAULT,
-	GPM_GRAPH_WIDGET_COLOUR_WHITE,
-	GPM_GRAPH_WIDGET_COLOUR_BLACK,
-	GPM_GRAPH_WIDGET_COLOUR_RED,
-	GPM_GRAPH_WIDGET_COLOUR_BLUE,
-	GPM_GRAPH_WIDGET_COLOUR_GREEN,
-	GPM_GRAPH_WIDGET_COLOUR_MAGENTA,
-	GPM_GRAPH_WIDGET_COLOUR_YELLOW,
-	GPM_GRAPH_WIDGET_COLOUR_CYAN,
-	GPM_GRAPH_WIDGET_COLOUR_GREY,
-	GPM_GRAPH_WIDGET_COLOUR_DARK_BLUE,
-	GPM_GRAPH_WIDGET_COLOUR_DARK_RED,
-	GPM_GRAPH_WIDGET_COLOUR_DARK_MAGENTA,
-	GPM_GRAPH_WIDGET_COLOUR_DARK_YELLOW,
-	GPM_GRAPH_WIDGET_COLOUR_DARK_GREEN,
-	GPM_GRAPH_WIDGET_COLOUR_DARK_CYAN,
-	GPM_GRAPH_WIDGET_COLOUR_DARK_GREY,
-	GPM_GRAPH_WIDGET_COLOUR_LAST
-} GpmGraphWidgetColour;
-
 typedef enum {
 	GPM_GRAPH_WIDGET_SHAPE_CIRCLE,
 	GPM_GRAPH_WIDGET_SHAPE_SQUARE,
@@ -80,13 +60,20 @@ typedef enum {
 	GPM_GRAPH_WIDGET_TYPE_LAST
 } GpmGraphWidgetAxisType;
 
+/* the different kinds of dots in the key */
 typedef struct {
 	guint			 id;
-	const gchar		*name;
-	GpmGraphWidgetColour	 colour;
+	guint32			 colour;
 	GpmGraphWidgetShape	 shape;
+	gchar			*desc;
 } GpmGraphWidgetKeyItem;
 
+/* the different kinds of lines in the key */
+typedef struct {
+	guint32			 colour;
+	gchar			*desc;
+} GpmGraphWidgetKeyData;
+
 struct GpmGraphWidget
 {
 	GtkDrawingArea	 parent;
@@ -110,18 +97,26 @@ void		 gpm_graph_widget_enable_events		(GpmGraphWidget	*graph,
 void		 gpm_graph_widget_set_invert_y		(GpmGraphWidget	*graph,
 							 gboolean	 inv);
 void		 gpm_graph_widget_set_data		(GpmGraphWidget	*graph,
-							 GList		*list);
+							 GpmArray	*array,
+							 guint		 id);
 void		 gpm_graph_widget_set_events		(GpmGraphWidget	*graph,
-							 GList		*list);
-void		 gpm_graph_widget_set_axis_x		(GpmGraphWidget	*graph,
+							 GpmArray	*array);
+void		 gpm_graph_widget_set_title		(GpmGraphWidget	*graph,
+							 const gchar	*title);
+void		 gpm_graph_widget_set_axis_type_x	(GpmGraphWidget	*graph,
 							 GpmGraphWidgetAxisType axis);
-void		 gpm_graph_widget_set_axis_y		(GpmGraphWidget	*graph,
+void		 gpm_graph_widget_set_axis_type_y	(GpmGraphWidget	*graph,
 							 GpmGraphWidgetAxisType axis);
-gboolean	 gpm_graph_widget_key_add		(GpmGraphWidget	*graph,
-							 const gchar	*name,
+gboolean	 gpm_graph_widget_key_data_clear	(GpmGraphWidget	*graph);
+gboolean	 gpm_graph_widget_key_data_add		(GpmGraphWidget	*graph,
+							 guint		 colour,
+							 const gchar	*desc);
+gboolean	 gpm_graph_widget_key_event_clear	(GpmGraphWidget	*graph);
+gboolean	 gpm_graph_widget_key_event_add		(GpmGraphWidget	*graph,
 							 guint		 id,
-							 GpmGraphWidgetColour colour,
-							 GpmGraphWidgetShape shape);
+							 guint32	 colour,
+							 GpmGraphWidgetShape shape,
+							 const gchar	*name);
 
 GpmGraphWidgetAxisType	 gpm_graph_widget_string_to_axis_type (const gchar	*type);
 
