@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2005-2007 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -58,7 +58,9 @@ typedef struct
 					    		 gboolean	 connected);
 	void		(* auth_request)		(GpmScreensaver	*screensaver,
 					    		 gboolean	 auth);
-	void		(* idle_changed)		(GpmScreensaver	*screensaver,
+	void		(* session_idle_changed)	(GpmScreensaver	*screensaver,
+					    		 gboolean	 is_idle);
+	void		(* powersave_idle_changed)	(GpmScreensaver	*screensaver,
 					    		 gboolean	 is_idle);
 } GpmScreensaverClass;
 
