@@ -30,6 +30,8 @@
 #include <math.h>
 #include <string.h>
 
+#include "gpm-array.h"
+#include "gpm-array-float.h"
 #include "gpm-common.h"
 #include "gpm-prefs.h"
 #include "gpm-conf.h"
@@ -37,24 +39,32 @@
 #include "gpm-debug.h"
 #include "gpm-stock-icons.h"
 #include "gpm-info.h"
-#include "gpm-info-data.h"
-#include "gpm-proxy.h"
-#include "gpm-powermanager.h"
+#include <libdbus-proxy.h>
 
 static void     gpm_statistics_class_init (GpmStatisticsClass *klass);
 static void     gpm_statistics_init       (GpmStatistics      *statistics);
 static void     gpm_statistics_finalize   (GObject	    *object);
 
+static void	gpm_statistics_refresh_data (GpmStatistics *statistics);
+
 #define GPM_STATISTICS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_STATISTICS, GpmStatisticsPrivate))
 
-#define ACTION_VOLTAGE		"voltage"
-#define ACTION_CHARGE		"charge"
-#define ACTION_POWER		"power"
-#define ACTION_TIME		"time"
-#define ACTION_CHARGE_TEXT	_("Charge History")
-#define ACTION_POWER_TEXT	_("Power History")
-#define ACTION_VOLTAGE_TEXT	_("Voltage History")
-#define ACTION_TIME_TEXT	_("Estimated Time History")
+#define ACTION_VOLTAGE				"voltage"
+#define ACTION_CHARGE				"charge"
+#define ACTION_POWER				"power"
+#define ACTION_TIME				"time"
+#define ACTION_PROFILE_CHARGE_TIME		"profile-charge-time"
+#define ACTION_PROFILE_CHARGE_ACCURACY		"profile-charge-accuracy"
+#define ACTION_PROFILE_DISCHARGE_TIME		"profile-discharge-time"
+#define ACTION_PROFILE_DISCHARGE_ACCURACY	"profile-discharge-accuracy"
+#define ACTION_CHARGE_TEXT			_("Charge History")
+#define ACTION_POWER_TEXT			_("Power History")
+#define ACTION_VOLTAGE_TEXT			_("Voltage History")
+#define ACTION_TIME_TEXT			_("Estimated Time History")
+#define ACTION_PROFILE_CHARGE_TIME_TEXT		_("Profiled Charge Time")
+#define ACTION_PROFILE_CHARGE_ACCURACY_TEXT	_("Profiled Charge Accuracy")
+#define ACTION_PROFILE_DISCHARGE_TIME_TEXT	_("Profiled Discharge Time")
+#define ACTION_PROFILE_DISCHARGE_ACCURACY_TEXT	_("Profiled Discharge Accuracy")
 
 #define GPM_STATISTICS_POLL_INTERVAL	15000 /* ms */
 
@@ -63,10 +73,12 @@ struct GpmStatisticsPrivate
 	GladeXML		*glade_xml;
 	GtkWidget		*graph_widget;
 	GpmConf			*conf;
-	GpmProxy		*gproxy;
-	GpmInfoData		*events;
-	GpmInfoData		*data;
+	DbusProxy		*gproxy;
+	GpmArray		*events;
+	GpmArray		*data;
 	const gchar		*graph_type;
+	gchar			*axis_desc_x;
+	gchar			*axis_desc_y;
 };
 
 enum {
@@ -171,8 +183,8 @@ conf_key_changed_cb (GpmConf       *conf,
 {
 	gboolean  enabled;
 
-	if (strcmp (key, GPM_CONF_AC_LOWPOWER) == 0) {
-		gpm_conf_get_bool (statistics->priv->conf, GPM_CONF_AC_LOWPOWER, &enabled);
+	if (strcmp (key, GPM_CONF_LOWPOWER_AC) == 0) {
+		gpm_conf_get_bool (statistics->priv->conf, GPM_CONF_LOWPOWER_AC, &enabled);
 		gpm_debug ("need to enable checkbox");
 	}
 }
@@ -214,16 +226,15 @@ gpm_statistics_get_events (GpmStatistics *statistics)
 	gint i;
 	gint x;
 	gint event;
-	gint time = 0;
 
 	g_return_val_if_fail (statistics != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_STATISTICS (statistics), FALSE);
 
-	proxy = gpm_proxy_get_proxy (statistics->priv->gproxy);
+	proxy = dbus_proxy_get_proxy (statistics->priv->gproxy);
 	if (proxy == NULL) {
 		gpm_warning ("not connected");
 		return FALSE;
-	}	
+	}
 
 	g_type_ptrarray = dbus_g_type_get_collection ("GPtrArray",
 					dbus_g_type_get_struct("GValueArray",
@@ -232,7 +243,6 @@ gpm_statistics_get_events (GpmStatistics *statistics)
 						G_TYPE_INVALID));
 
 	ret = dbus_g_proxy_call (proxy, "GetEventLog", &error,
-				 G_TYPE_INT, time,
 				 G_TYPE_INVALID,
 				 g_type_ptrarray, &ptrarray,
 				 G_TYPE_INVALID);
@@ -249,7 +259,7 @@ gpm_statistics_get_events (GpmStatistics *statistics)
 	gpm_debug ("events size=%i", ptrarray->len);
 
 	/* clear current events */
-	gpm_info_data_clear (statistics->priv->events);
+	gpm_array_clear (statistics->priv->events);
 
 	for (i=0; i< ptrarray->len; i++) {
 		gva = (GValueArray *) g_ptr_array_index (ptrarray, i);
@@ -259,7 +269,7 @@ gpm_statistics_get_events (GpmStatistics *statistics)
 		gv = g_value_array_get_nth (gva, 1);
 		event = g_value_get_int (gv);
 		g_value_unset (gv);
-		gpm_info_data_add_always (statistics->priv->events, x, event, 0, NULL);
+		gpm_array_append (statistics->priv->events, x, event, 0);
 		g_value_array_free (gva);
 	}
 	g_ptr_array_free (ptrarray, TRUE);
@@ -270,12 +280,9 @@ gpm_statistics_get_events (GpmStatistics *statistics)
 static void
 gpm_statistics_refresh_events (GpmStatistics *statistics)
 {
-	GList *list;
-
 	gpm_statistics_get_events (statistics);
-	list = gpm_info_data_get_list (statistics->priv->events);
-
-	gpm_graph_widget_set_events (GPM_GRAPH_WIDGET (statistics->priv->graph_widget), list);
+	gpm_graph_widget_set_events (GPM_GRAPH_WIDGET (statistics->priv->graph_widget),
+				     statistics->priv->events);
 }
 
 /**
@@ -292,59 +299,51 @@ gpm_statistics_checkbox_events_cb (GtkWidget     *widget,
 	gpm_debug ("Events enable %i", checked);
 
 	/* save to gconf so we open next time with the correct setting */
-	gpm_conf_set_bool (statistics->priv->conf, GPM_CONF_STAT_SHOW_EVENTS, checked);
+	gpm_conf_set_bool (statistics->priv->conf, GPM_CONF_STATS_SHOW_EVENTS, checked);
 
 	if (checked == FALSE) {
 		/* remove the dots from the graph */
 		gpm_graph_widget_enable_events (GPM_GRAPH_WIDGET (statistics->priv->graph_widget), FALSE);
+		/* disable legend  */
+		gpm_graph_widget_enable_legend (GPM_GRAPH_WIDGET (statistics->priv->graph_widget), FALSE);
 		return;
 	}
 
-	/* refresh data automatically */
+	/* refresh events automatically */
 	gpm_statistics_refresh_events (statistics);
 
 	/* only enable the dots if the checkbox is checked */
 	gpm_graph_widget_enable_events (GPM_GRAPH_WIDGET (statistics->priv->graph_widget), TRUE);
+
+	/* enable legend  */
+	gpm_graph_widget_enable_legend (GPM_GRAPH_WIDGET (statistics->priv->graph_widget), TRUE);
 }
 
 /**
- * gpm_statistics_checkbox_legend_cb:
- * @widget: The GtkWidget object
- * @gpm_pref_key: The GConf key for this preference setting.
+ * gpm_statistics_refresh_axis_labels:
  **/
 static void
-gpm_statistics_checkbox_legend_cb (GtkWidget *widget,
-			    GpmStatistics  *statistics)
+gpm_statistics_refresh_axis_labels (GpmStatistics *statistics)
 {
-	gboolean checked;
-
-	checked = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
-	gpm_debug ("Legend enable %i", checked);
+	gboolean show;
+	GtkWidget *widget1;
+	GtkWidget *widget2;
 
 	/* save to gconf so we open next time with the correct setting */
-	gpm_conf_set_bool (statistics->priv->conf, GPM_CONF_STAT_SHOW_LEGEND, checked);
+	gpm_conf_get_bool (statistics->priv->conf, GPM_CONF_STATS_SHOW_AXIS_LABELS, &show);
 
-	gpm_graph_widget_enable_legend (GPM_GRAPH_WIDGET (statistics->priv->graph_widget), checked);
-}
+	widget1 = glade_xml_get_widget (statistics->priv->glade_xml, "label_x_axis");
+	widget2 = glade_xml_get_widget (statistics->priv->glade_xml, "label_y_axis");
 
-/**
- * gpm_statistics_checkbox_axis_labels_cb:
- * @widget: The GtkWidget object
- * @gpm_pref_key: The GConf key for this preference setting.
- **/
-static void
-gpm_statistics_checkbox_axis_labels_cb (GtkWidget *widget,
-			    GpmStatistics  *statistics)
-{
-	gboolean checked;
-
-	checked = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
-	gpm_debug ("axis labels enable %i", checked);
-
-	/* save to gconf so we open next time with the correct setting */
-	gpm_conf_set_bool (statistics->priv->conf, GPM_CONF_STAT_SHOW_AXIS_LABELS, checked);
-
-	gpm_graph_widget_enable_axis_labels (GPM_GRAPH_WIDGET (statistics->priv->graph_widget), checked);
+	if (show == FALSE) {
+		gtk_widget_hide (widget1);
+		gtk_widget_hide (widget2);
+	} else {
+		gtk_label_set_text (GTK_LABEL (widget1), statistics->priv->axis_desc_x);
+		gtk_label_set_text (GTK_LABEL (widget2), statistics->priv->axis_desc_y);
+		gtk_widget_show (widget1);
+		gtk_widget_show (widget2);
+	}
 }
 
 /**
@@ -374,11 +373,11 @@ gpm_statistics_find_types (GpmStatistics *statistics,
 	gchar **strlist;
 	DBusGProxy *proxy;
 
-	proxy = gpm_proxy_get_proxy (statistics->priv->gproxy);
+	proxy = dbus_proxy_get_proxy (statistics->priv->gproxy);
 	if (proxy == NULL) {
 		gpm_warning ("not connected");
 		return FALSE;
-	}	
+	}
 
 	ret = dbus_g_proxy_call (proxy, "GetDataTypes", &error,
 			         G_TYPE_INVALID,
@@ -431,17 +430,16 @@ gpm_statistics_get_data_dbus (GpmStatistics *statistics,
 	GType g_type_ptrarray;
 	int i;
 	int x, y, col;
-	gint time = 0;
 
 	g_return_val_if_fail (statistics != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_STATISTICS (statistics), FALSE);
 	g_return_val_if_fail (type != NULL, FALSE);
 
-	proxy = gpm_proxy_get_proxy (statistics->priv->gproxy);
+	proxy = dbus_proxy_get_proxy (statistics->priv->gproxy);
 	if (proxy == NULL) {
 		gpm_warning ("not connected");
 		return FALSE;
-	}	
+	}
 
 	g_type_ptrarray = dbus_g_type_get_collection ("GPtrArray",
 					dbus_g_type_get_struct("GValueArray",
@@ -451,7 +449,6 @@ gpm_statistics_get_data_dbus (GpmStatistics *statistics,
 						G_TYPE_INVALID));
 
 	ret = dbus_g_proxy_call (proxy, "GetData", &error,
-				 G_TYPE_INT, time,
 				 G_TYPE_STRING, type,
 				 G_TYPE_INVALID,
 				 g_type_ptrarray, &ptrarray,
@@ -469,7 +466,7 @@ gpm_statistics_get_data_dbus (GpmStatistics *statistics,
 	gpm_debug ("size=%i", ptrarray->len);
 
 	/* clear current events */
-	gpm_info_data_clear (statistics->priv->data);
+	gpm_array_clear (statistics->priv->data);
 
 	for (i=0; i< ptrarray->len; i++) {
 		gva = (GValueArray *) g_ptr_array_index (ptrarray, i);
@@ -482,7 +479,7 @@ gpm_statistics_get_data_dbus (GpmStatistics *statistics,
 		gv = g_value_array_get_nth (gva, 2);
 		col = g_value_get_int (gv);
 		g_value_unset (gv);
-		gpm_info_data_add_always (statistics->priv->data, x, y, col, NULL);
+		gpm_array_append (statistics->priv->data, x, y, col);
 		g_value_array_free (gva);
 	}
 	g_ptr_array_free (ptrarray, TRUE);
@@ -491,33 +488,52 @@ gpm_statistics_get_data_dbus (GpmStatistics *statistics,
 }
 
 static gboolean
-gpm_statistics_get_axis_type_dbus (GpmStatistics          *statistics,
-			      const gchar 	     *type,
-			      GpmGraphWidgetAxisType *x,
-			      GpmGraphWidgetAxisType *y)
+gpm_statistics_get_parameters_dbus (GpmStatistics *statistics,
+			            const gchar   *type)
 {
 	GError *error = NULL;
 	gboolean ret;
-	gchar *axis_type_x;
-	gchar *axis_type_y;
+	GpmGraphWidgetAxisType axis_type_x;
+	GpmGraphWidgetAxisType axis_type_y;
+	gchar *axis_type_x_text;
+	gchar *axis_type_y_text;
 	DBusGProxy *proxy;
+	GPtrArray *ptr_data_array = NULL;
+	GPtrArray *ptr_event_array = NULL;
+	GType g_type_ptr_data_array;
+	GType g_type_ptr_event_array;
 
 	g_return_val_if_fail (statistics != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_STATISTICS (statistics), FALSE);
-	g_return_val_if_fail (x != NULL, FALSE);
-	g_return_val_if_fail (y != NULL, FALSE);
 
-	proxy = gpm_proxy_get_proxy (statistics->priv->gproxy);
+	g_type_ptr_data_array = dbus_g_type_get_collection ("GPtrArray",
+					dbus_g_type_get_struct("GValueArray",
+						G_TYPE_INT,
+						G_TYPE_STRING,
+						G_TYPE_INVALID));
+	g_type_ptr_event_array = dbus_g_type_get_collection ("GPtrArray",
+					dbus_g_type_get_struct("GValueArray",
+						G_TYPE_INT,
+						G_TYPE_INT,
+						G_TYPE_INT,
+						G_TYPE_STRING,
+						G_TYPE_INVALID));
+
+	proxy = dbus_proxy_get_proxy (statistics->priv->gproxy);
 	if (proxy == NULL) {
 		gpm_warning ("not connected");
 		return FALSE;
-	}	
+	}
 
-	ret = dbus_g_proxy_call (proxy, "GetAxisTypes", &error,
+	ret = dbus_g_proxy_call (proxy, "GetParameters", &error,
 			         G_TYPE_STRING, type,
 			         G_TYPE_INVALID,
-			         G_TYPE_STRING, &axis_type_x,
-			         G_TYPE_STRING, &axis_type_y,
+			         G_TYPE_STRING, &axis_type_x_text,
+			         G_TYPE_STRING, &axis_type_y_text,
+			         G_TYPE_STRING, &statistics->priv->axis_desc_x,
+			         G_TYPE_STRING, &statistics->priv->axis_desc_y,
+				 g_type_ptr_data_array, &ptr_data_array,
+				 g_type_ptr_event_array, &ptr_event_array,
 			         G_TYPE_INVALID);
 	if (error) {
 		gpm_debug ("ERROR: %s", error->message);
@@ -529,12 +545,64 @@ gpm_statistics_get_axis_type_dbus (GpmStatistics          *statistics,
 		return FALSE;
 	}
 
-	gpm_debug ("graph type '%s' mapped to x-axis '%s'", type, axis_type_x);
-	gpm_debug ("graph type '%s' mapped to y-axis '%s'", type, axis_type_y);
+	guint i;
+	gint id;
+	gint colour;
+	gint shape;
+	GValueArray *gva;
+	GValue *gv;
+	const gchar *desc;
+
+	/* clear the data key */
+	gpm_graph_widget_key_data_clear (GPM_GRAPH_WIDGET (statistics->priv->graph_widget));
+
+	for (i=0; i< ptr_data_array->len; i++) {
+		gva = (GValueArray *) g_ptr_array_index (ptr_data_array, i);
+		gv = g_value_array_get_nth (gva, 0);
+		id = g_value_get_int (gv);
+		g_value_unset (gv);
+		gv = g_value_array_get_nth (gva, 1);
+		desc = g_value_get_string (gv);
+		/* add to the data key */
+		gpm_graph_widget_key_data_add (GPM_GRAPH_WIDGET (statistics->priv->graph_widget), id, desc);
+		g_value_unset (gv);
+		g_value_array_free (gva);
+	}
+	g_ptr_array_free (ptr_data_array, TRUE);
+
+	/* clear the events key */
+	gpm_graph_widget_key_event_clear (GPM_GRAPH_WIDGET (statistics->priv->graph_widget));
+
+	/* process events key */
+	for (i=0; i< ptr_event_array->len; i++) {
+		gva = (GValueArray *) g_ptr_array_index (ptr_event_array, i);
+		gv = g_value_array_get_nth (gva, 0);
+		id = g_value_get_int (gv);
+		g_value_unset (gv);
+		gv = g_value_array_get_nth (gva, 1);
+		colour = g_value_get_int (gv);
+		g_value_unset (gv);
+		gv = g_value_array_get_nth (gva, 2);
+		shape = g_value_get_int (gv);
+		g_value_unset (gv);
+		gv = g_value_array_get_nth (gva, 3);
+		desc = g_value_get_string (gv);
+		/* add to the data key */
+		gpm_graph_widget_key_event_add (GPM_GRAPH_WIDGET (statistics->priv->graph_widget), id, colour, shape, desc);
+		g_value_unset (gv);
+		g_value_array_free (gva);
+	}
+	g_ptr_array_free (ptr_event_array, TRUE);
+
+	gpm_debug ("graph type '%s' mapped to x-axis '%s'", type, axis_type_x_text);
+	gpm_debug ("graph type '%s' mapped to y-axis '%s'", type, axis_type_y_text);
 
 	/* convert the string types to enumerated values */
-	*x = gpm_graph_widget_string_to_axis_type (axis_type_x);
-	*y = gpm_graph_widget_string_to_axis_type (axis_type_y);
+	axis_type_x = gpm_graph_widget_string_to_axis_type (axis_type_x_text);
+	axis_type_y = gpm_graph_widget_string_to_axis_type (axis_type_y_text);
+
+	gpm_graph_widget_set_axis_type_x (GPM_GRAPH_WIDGET (statistics->priv->graph_widget), axis_type_x);
+	gpm_graph_widget_set_axis_type_y (GPM_GRAPH_WIDGET (statistics->priv->graph_widget), axis_type_y);
 
 	return TRUE;
 }
@@ -542,16 +610,33 @@ gpm_statistics_get_axis_type_dbus (GpmStatistics          *statistics,
 static void
 gpm_statistics_refresh_data (GpmStatistics *statistics)
 {
-	GList *list = NULL;
+	gboolean smooth;
 
 	/* only get the data for a valid type */
 	if (statistics->priv->graph_type != NULL) {
 		gpm_statistics_get_data_dbus (statistics, statistics->priv->graph_type);
 	}
 
-	list = gpm_info_data_get_list (statistics->priv->data);
+	gpm_conf_get_bool (statistics->priv->conf, GPM_CONF_STATS_SMOOTH_DATA, &smooth);
+	if (smooth == TRUE) {
+		GArray *arrayfloat;
+		GArray *kernel;
+		GArray *result;
+		gpm_debug ("smoothing data, slooooooow....");
 
-	gpm_graph_widget_set_data (GPM_GRAPH_WIDGET (statistics->priv->graph_widget), list, 0);
+		arrayfloat = gpm_array_float_new (gpm_array_get_size (statistics->priv->data));
+		gpm_array_float_from_array_y (arrayfloat, statistics->priv->data);
+		kernel = gpm_array_float_compute_gaussian (17, 4.0);
+		result = gpm_array_float_convolve (arrayfloat, kernel);
+		gpm_array_float_to_array_y (result, statistics->priv->data);
+
+		gpm_array_float_free (kernel);
+		gpm_array_float_free (arrayfloat);
+		gpm_array_float_free (result);
+	}
+
+	gpm_graph_widget_set_data (GPM_GRAPH_WIDGET (statistics->priv->graph_widget),
+				   statistics->priv->data, 0);
 
 	gtk_widget_hide (GTK_WIDGET (statistics->priv->graph_widget));
 	gtk_widget_show (GTK_WIDGET (statistics->priv->graph_widget));
@@ -562,9 +647,7 @@ gpm_statistics_type_combo_changed_cb (GtkWidget      *widget,
 				      GpmStatistics  *statistics)
 {
 	gchar *value;
-	gchar *type;
-	GpmGraphWidgetAxisType axis_x = GPM_GRAPH_WIDGET_TYPE_INVALID;
-	GpmGraphWidgetAxisType axis_y = GPM_GRAPH_WIDGET_TYPE_INVALID;
+	gchar *type = NULL;
 
 	value = gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget));
 	if (value == NULL) {
@@ -579,25 +662,33 @@ gpm_statistics_type_combo_changed_cb (GtkWidget      *widget,
 		type = ACTION_TIME;
 	} else if (strcmp (value, ACTION_VOLTAGE_TEXT) == 0) {
 		type = ACTION_VOLTAGE;
+	} else if (strcmp (value, ACTION_PROFILE_CHARGE_TIME_TEXT) == 0) {
+		type = ACTION_PROFILE_CHARGE_TIME;
+	} else if (strcmp (value, ACTION_PROFILE_CHARGE_ACCURACY_TEXT) == 0) {
+		type = ACTION_PROFILE_CHARGE_ACCURACY;
+	} else if (strcmp (value, ACTION_PROFILE_DISCHARGE_TIME_TEXT) == 0) {
+		type = ACTION_PROFILE_DISCHARGE_TIME;
+	} else if (strcmp (value, ACTION_PROFILE_DISCHARGE_ACCURACY_TEXT) == 0) {
+		type = ACTION_PROFILE_DISCHARGE_ACCURACY;
 	} else {
-		gpm_debug ("type not known, ignoring");
-		return;
+		g_error ("value '%s' unknown", value);
 	}
 	g_free (value);
 
 	/* find out what sort of grid axis we need */
-	gpm_statistics_get_axis_type_dbus (statistics, type, &axis_x, &axis_y);
-	gpm_graph_widget_set_axis_type_x (GPM_GRAPH_WIDGET (statistics->priv->graph_widget), axis_x);
-	gpm_graph_widget_set_axis_type_y (GPM_GRAPH_WIDGET (statistics->priv->graph_widget), axis_y);
+	gpm_statistics_get_parameters_dbus (statistics, type);
 
 	/* const, so no need to free */
 	statistics->priv->graph_type = type;
 
 	/* save in gconf so we choose the correct graph type on next startup */
-	gpm_conf_set_string (statistics->priv->conf, GPM_CONF_STAT_GRAPH_TYPE, type);
+	gpm_conf_set_string (statistics->priv->conf, GPM_CONF_STATS_GRAPH_TYPE, type);
 
 	/* refresh data automatically */
 	gpm_statistics_refresh_data (statistics);
+
+	/* refresh the axis text */
+	gpm_statistics_refresh_axis_labels (statistics);
 }
 
 static void
@@ -617,7 +708,7 @@ gpm_statistics_populate_graph_types (GpmStatistics *statistics,
 		return;
 	}
 
-	gpm_conf_get_string (statistics->priv->conf, GPM_CONF_STAT_GRAPH_TYPE, &saved);
+	gpm_conf_get_string (statistics->priv->conf, GPM_CONF_STATS_GRAPH_TYPE, &saved);
 	/* gconf error, bahh */
 	if (saved == NULL) {
 		saved = g_strdup ("power");
@@ -635,6 +726,14 @@ gpm_statistics_populate_graph_types (GpmStatistics *statistics,
 			type_localized = ACTION_TIME_TEXT;
 		} else if (strcmp (type, ACTION_VOLTAGE) == 0) {
 			type_localized = ACTION_VOLTAGE_TEXT;
+		} else if (strcmp (type, ACTION_PROFILE_CHARGE_TIME) == 0) {
+			type_localized = ACTION_PROFILE_CHARGE_TIME_TEXT;
+		} else if (strcmp (type, ACTION_PROFILE_CHARGE_ACCURACY) == 0) {
+			type_localized = ACTION_PROFILE_CHARGE_ACCURACY_TEXT;
+		} else if (strcmp (type, ACTION_PROFILE_DISCHARGE_TIME) == 0) {
+			type_localized = ACTION_PROFILE_DISCHARGE_TIME_TEXT;
+		} else if (strcmp (type, ACTION_PROFILE_DISCHARGE_ACCURACY) == 0) {
+			type_localized = ACTION_PROFILE_DISCHARGE_ACCURACY_TEXT;
 		} else {
 			type_localized = _("Unknown");
 		}
@@ -654,10 +753,11 @@ gpm_statistics_populate_graph_types (GpmStatistics *statistics,
 	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), pos);
 }
 
-static gboolean 
+static gboolean
 gpm_statistics_graph_refresh (gpointer data)
 {
 	GpmStatistics *statistics = GPM_STATISTICS (data);
+
 	gpm_debug ("refreshing graph type '%s'", statistics->priv->graph_type);
 	gpm_statistics_refresh_data (statistics);
 	gpm_statistics_refresh_events (statistics);
@@ -697,22 +797,21 @@ gpm_statistics_init (GpmStatistics *statistics)
 
 	glade_set_custom_handler (gpm_graph_widget_custom_handler, statistics);
 
-	statistics->priv->gproxy = gpm_proxy_new ();
-	gpm_proxy_assign (statistics->priv->gproxy,
-			  GPM_PROXY_SESSION,
+	statistics->priv->gproxy = dbus_proxy_new ();
+	dbus_proxy_assign (statistics->priv->gproxy,
+			  DBUS_PROXY_SESSION,
 			  GPM_DBUS_SERVICE,
 			  GPM_DBUS_PATH_STATS,
 			  GPM_DBUS_INTERFACE_STATS);
 
 	/* would happen if not using g-p-m or using an old version of g-p-m */
-	if (gpm_proxy_is_connected (statistics->priv->gproxy) == FALSE) {
-		gpm_critical_error ("%s\n%s", _("Could not connect to GNOME Power Manager."),
-				    _("Perhaps the program is not running or you are using an old version?"));
+	if (dbus_proxy_is_connected (statistics->priv->gproxy) == FALSE) {
+		gpm_error (_("Could not connect to GNOME Power Manager."));
 	}
 
 	statistics->priv->graph_type = NULL;
-	statistics->priv->events = gpm_info_data_new ();
-	statistics->priv->data = gpm_info_data_new ();
+	statistics->priv->events = gpm_array_new ();
+	statistics->priv->data = gpm_array_new ();
 
 	statistics->priv->glade_xml = glade_xml_new (GPM_DATA "/gpm-graph.glade", NULL, NULL);
 
@@ -741,79 +840,6 @@ gpm_statistics_init (GpmStatistics *statistics)
 	gpm_graph_widget_set_axis_type_x (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_TYPE_TIME);
 	gpm_graph_widget_set_axis_type_y (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_TYPE_PERCENTAGE);
 
-	/* add the key items */
-	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
-				  _("On AC"),
-				  GPM_EVENT_ON_AC,
-				  GPM_GRAPH_WIDGET_COLOUR_BLUE,
-				  GPM_GRAPH_WIDGET_SHAPE_CIRCLE);
-	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
-				  _("On battery"),
-				  GPM_EVENT_ON_BATTERY,
-				  GPM_GRAPH_WIDGET_COLOUR_DARK_BLUE,
-				  GPM_GRAPH_WIDGET_SHAPE_CIRCLE);
-	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
-				  _("Session idle"),
-				  GPM_EVENT_SESSION_IDLE,
-				  GPM_GRAPH_WIDGET_COLOUR_YELLOW,
-				  GPM_GRAPH_WIDGET_SHAPE_SQUARE);
-	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
-				  _("Session active"),
-				  GPM_EVENT_SESSION_ACTIVE,
-				  GPM_GRAPH_WIDGET_COLOUR_DARK_YELLOW,
-				  GPM_GRAPH_WIDGET_SHAPE_SQUARE);
-	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
-				  _("Suspend"),
-				  GPM_EVENT_SUSPEND,
-				  GPM_GRAPH_WIDGET_COLOUR_RED,
-				  GPM_GRAPH_WIDGET_SHAPE_DIAMOND);
-	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
-				  _("Resume"),
-				  GPM_EVENT_RESUME,
-				  GPM_GRAPH_WIDGET_COLOUR_DARK_RED,
-				  GPM_GRAPH_WIDGET_SHAPE_DIAMOND);
-	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
-				  _("Hibernate"),
-				  GPM_EVENT_HIBERNATE,
-				  GPM_GRAPH_WIDGET_COLOUR_MAGENTA,
-				  GPM_GRAPH_WIDGET_SHAPE_DIAMOND);
-	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
-				  _("Lid closed"),
-				  GPM_EVENT_LID_CLOSED,
-				  GPM_GRAPH_WIDGET_COLOUR_GREEN,
-				  GPM_GRAPH_WIDGET_SHAPE_TRIANGLE);
-	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
-				  _("Lid opened"),
-				  GPM_EVENT_LID_OPENED,
-				  GPM_GRAPH_WIDGET_COLOUR_DARK_GREEN,
-				  GPM_GRAPH_WIDGET_SHAPE_TRIANGLE);
-	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
-				  _("Notification"),
-				  GPM_EVENT_NOTIFICATION,
-				  GPM_GRAPH_WIDGET_COLOUR_GREY,
-				  GPM_GRAPH_WIDGET_SHAPE_CIRCLE);
-	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
-				  _("DPMS On"),
-				  GPM_EVENT_DPMS_ON,
-				  GPM_GRAPH_WIDGET_COLOUR_CYAN,
-				  GPM_GRAPH_WIDGET_SHAPE_CIRCLE);
-	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
-				  _("DPMS Standby"),
-				  GPM_EVENT_DPMS_STANDBY,
-				  GPM_GRAPH_WIDGET_COLOUR_CYAN,
-				  GPM_GRAPH_WIDGET_SHAPE_TRIANGLE);
-	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
-				  _("DPMS Suspend"),
-				  GPM_EVENT_DPMS_SUSPEND,
-				  GPM_GRAPH_WIDGET_COLOUR_CYAN,
-				  GPM_GRAPH_WIDGET_SHAPE_SQUARE);
-	gpm_graph_widget_key_add (GPM_GRAPH_WIDGET (widget),
-				  _("DPMS Off"),
-				  GPM_EVENT_DPMS_OFF,
-				  GPM_GRAPH_WIDGET_COLOUR_CYAN,
-				  GPM_GRAPH_WIDGET_SHAPE_DIAMOND);
-				       
-
 	/* FIXME: There's got to be a better way than this */
 	gtk_widget_hide (GTK_WIDGET (widget));
 	gtk_widget_show (GTK_WIDGET (widget));
@@ -821,31 +847,15 @@ gpm_statistics_init (GpmStatistics *statistics)
 	widget = glade_xml_get_widget (statistics->priv->glade_xml, "combobox_type");
 	gpm_statistics_populate_graph_types (statistics, widget);
 
-	widget = glade_xml_get_widget (statistics->priv->glade_xml, "combobox_device");
-	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), "Default");
-	gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 0);
-	gtk_widget_set_sensitive (GTK_WIDGET (widget), FALSE);
-
 	widget = glade_xml_get_widget (statistics->priv->glade_xml, "checkbutton_events");
-	gpm_conf_get_bool (statistics->priv->conf, GPM_CONF_STAT_SHOW_EVENTS, &checked);
+	gpm_conf_get_bool (statistics->priv->conf, GPM_CONF_STATS_SHOW_EVENTS, &checked);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), checked);
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (gpm_statistics_checkbox_events_cb), statistics);
 	gpm_statistics_checkbox_events_cb (widget, statistics);
 
-	widget = glade_xml_get_widget (statistics->priv->glade_xml, "checkbutton_legend");
-	gpm_conf_get_bool (statistics->priv->conf, GPM_CONF_STAT_SHOW_LEGEND, &checked);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), checked);
-	g_signal_connect (widget, "clicked",
-			  G_CALLBACK (gpm_statistics_checkbox_legend_cb), statistics);
-	gpm_statistics_checkbox_legend_cb (widget, statistics);
-
-	widget = glade_xml_get_widget (statistics->priv->glade_xml, "checkbutton_axis");
-	gpm_conf_get_bool (statistics->priv->conf, GPM_CONF_STAT_SHOW_AXIS_LABELS, &checked);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), checked);
-	g_signal_connect (widget, "clicked",
-			  G_CALLBACK (gpm_statistics_checkbox_axis_labels_cb), statistics);
-	gpm_statistics_checkbox_axis_labels_cb (widget, statistics);
+	/* refresh the axis */
+	gpm_statistics_refresh_axis_labels (statistics);
 
 	gtk_widget_show (main_window);
 	g_timeout_add (GPM_STATISTICS_POLL_INTERVAL,
