@@ -2,7 +2,7 @@
  *
  * Copyright (C) 2002 Jorn Baayen
  * Copyright (C) 2003,2004 Colin Walters <walters@verbum.org>
- * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2005-2007 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -31,7 +31,9 @@ G_BEGIN_DECLS
 #define GPM_STOCK_BATTERY_CHARGED		"gpm-primary-charged"
 #define GPM_STOCK_HIBERNATE			"gpm-hibernate"
 #define GPM_STOCK_SUSPEND			"gpm-suspend"
-#define GPM_STOCK_BRIGHTNESS			"gpm-brightness"
+#define GPM_STOCK_STATISTICS			"gpm-statistics"
+#define GPM_STOCK_BRIGHTNESS_LCD		"gpm-brightness-lcd"
+#define GPM_STOCK_BRIGHTNESS_KBD		"gpm-brightness-kbd"
 
 gboolean	gpm_stock_icons_init			(void);
 void		gpm_stock_icons_shutdown		(void);
